/* RTC & SHT-11 I2C-Program Test-Program
   Program By GucciTech - Chojiro Higashiguchi
   Date 2008/08/03
   Program Name 'SHT_RTC.C'
   PIC MPU PIC16F877-20/SP
   LCD = PWB24227-CEM
   RTC-Chip = RTC-8564NB
   SENCER = SHT11
   
   I/O Pin Layout
      PORT-B0     Edit-SW
      PORT-B1     Up-SW
      PORT-B2     Down-SW
      PORT-B3     N.C
      PORT-B4     Right-SW
      PORT-B5     Left-SW
      PORT-B6     ICD-U64 Debuger PGC
      PORT-B7     ICD-U64 Debuger PGD

      PORT-C0     MCP3208 CS
      PORT-C1     LED_Red
      PORT-C2     N.C
      PORT-C3     I2C-SCL
      PORT-C4     I2C-SDA
      PORT-C5     MCP3208 DIN
      PORT-C6     RxD
      PORT-C7     TxD

      PORT-D0     LCD-E
      PORT-D1     LCD-RS
      PORT-D2     N.C
      PORT-D3     N.C
      PORT-D4     LCD-DB0
      PORT-D5     LCD-DB1
      PORT-D6     LCD-DB2
      PORT-D7     LCD-DB3

      PORT-E0     SHT_SCK
      PORT-E1     SHT_DAT
      PORT-E2     N.C
*/

#include "SHT_RTC.h"
#include "lcd_lib_G1.c"
#include "SHT_lib.c"
#include "rtc_lib.c"

void time_disp1();            // Now Date & Time Disp1
void time_disp2();            // First Data & Time Disp2
void Up_value(int Upval);     // 'Up' Switch Push Routine
void Down_value(int Dnval);   // 'Down' Switch Push Routine
void sht_sampling();          // 'Temperature' & 'Humidity' Data Sampling
void sht_disp();              // 'Temperature' & 'Humidity' Data Display

static unsigned int  year_h2;       // Year ??xx Data Buffer
static unsigned int  UpDn_sw_cnt;   // Up-Dwon Switch Counter Buffer
static unsigned int  RL_sw_cnt;     // Right-Left Switch Counter Buffer
static unsigned int  sec_Buf;
static unsigned int  min_Buf;
static unsigned int  hour_Buf;
static unsigned int  day_Buf;
static unsigned int  week_Buf;
static unsigned int  month_Buf;
static unsigned int  year_Buf;
static unsigned short century;
static unsigned short change_1;     // Switch Chenge Check Flag
static unsigned short one_secf;     // 1sec Flag

static float tval;
static float rhval;
static float x;
static unsigned long rdata_y;

int const MonthDay[12]={
   0x31,0x29,0x31,0x30,0x31,0x30,0x31,0x31,0x30,0x31,0x30,0x31};
int const CurPosi[7]={
   0x83,0x86,0x89,0x8d,0x90,0x93,0x96};

static char d_week[7][4]=
{
   {"Sun"},
   {"Mon"},
   {"Tue"},
   {"Wed"},
   {"Thu"},
   {"Fri"},
   {"Sat"}
};

void sht_sampling()
{
   // "Transmission Start" sequence
   SHT_Trans_Start();
   // *******************************
   // Temperature measurement command
   // *******************************
   SHT_Trans(0b00000011);    // Command transmission
   // SHT Time to measure temperature by 14bit accuracy
   // 210ms * 1.15 = 315ms
   delay_ms(210);
   // End Conversion check
   while(input(SHT_DAT) == 1);
   // SHT data recived
   SHT_data_recive();
   rdata_y = rdata_x;
   // Temperature = d1 + d2 x rdata_y
   // d1 = -40 (VDD = 5V), d2 = 0.01 (14bit)
   tval = (float)rdata_y * 0.01 - 40.0;

   // "Transmission Start" sequence
   SHT_Trans_Start();
   // *******************************
   // Humidity measurement command
   // *******************************
   SHT_Trans(0b00000101);     // Command transmission
   // Time to measure temperature by 12bit accuracy
   // 55ms
   delay_ms(210);
   // End Conversion check
   while(input(SHT_DAT) == 1);
   // SHT data recived
   SHT_data_recive();
   rdata_y = rdata_x;

   // RH = C1 + C2 x rdata_y + C3 x rdata_y x rdata_y
   x = (float)rdata_y;
   rhval = -4 + 0.0405 * x - 0.0000028 * x * x;
}

void sht_disp()
{
   // *******************************
   // DATA output
   // *******************************
   static unsigned int i;
   // Temperature & Humidity output
   i = 0xDF;                  // ASCII 'do'-Mark
   LCD_cmd(0xc0);             // move to line 2
   printf(LCD_data, "T%3.1f%CC RH%3.1f%% P1013hP",tval,i,rhval);
}

void time_disp1()
{
   rtc_date_read();           // RTC-8564NB Data Read
   if ((l_week >= 0x30) && (l_week <= 0x36))
   {
      if (sec_Buf != sec)
      {
         lcd_cmd(0x94);
         printf(lcd_data,":%c%c",h_sec,l_sec);
         sec_Buf = sec;
         one_secf = 1;        // 1sec Flag '1' set
      }
      if (min_Buf != min)
      {
         lcd_cmd(0x91);
         printf(lcd_data,":%c%c",h_min,l_min);
         min_Buf = min;
      }
      if (hour_Buf != hour)
      {
         lcd_cmd(0x8f);
         printf(lcd_data,"%c%c",h_hour,l_hour);
         hour_Buf = hour;
      }
      if (week_Buf != week)
      {
         lcd_cmd(0x8b);
         printf(lcd_data,"%s",d_week[l_week - 0x30]);
         week_Buf = week;
      }
      if (day_Buf != day)
      {
         lcd_cmd(0x87);
         printf(lcd_data,"/%c%c",h_day,l_day);
         day_Buf = day;
      }
      if (month_Buf != (month & 0x1f))
      {
         lcd_cmd(0x84);
         printf(lcd_data,"/%c%c",h_month,l_month);
         month_Buf = month;
      }
      if (year_Buf != year)
      {
         if ((month & 0x80) == 1)
            century = 1;
         else
            century = 0;
         lcd_cmd(0x80);
         printf(lcd_data,"%u%c%c",year_h2,h_year,l_year);
         year_Buf = year;
      }
   }
}

void time_disp2()
{
   rtc_date_read();           // RTC-8564NB Data Read
   if ((l_week >= 0x30) && (l_week <= 0x36))
   {
      lcd_cmd(0x80);          //LCD 1-Line Head
      printf(lcd_data,"%u%c%c/%c%c/%c%c %s",year_h2,h_year,l_year,h_month,l_month,h_day,l_day,d_week[l_week - 0x30]);
      lcd_cmd(0x8f);          //LCD 1-Line Time-Data Head
      printf(lcd_data,"%c%c:%c%c:%c%c",h_hour,l_hour,h_min,l_min,h_sec,l_sec);
   }
}

void Up_value(int Upval)
{
   switch(Upval)
   {
      case 0:                 //Year Valu Data Increment
      {
         year++;
         if ((year & 0x0f) == 0x0a)
            year = ((year + 0x10) & 0xf0);
         if ((year & 0xf0) == 0xa0)
         {
            year = year & 0x0f;
            year_h2++;
         }
         break;
      }
      case 1:                 //Month Value Data Increment
      {
         month++;
         if ((month & 0x0f) == 0x0a)
            month = ((month + 0x10) & 0xf0);
         if (month >= 0x13 )
            month = 0x01;
         break;
      }
      case 2:                 //Day Value Data Increment
      {
         day++;
         if ((day & 0x0f) == 0x0a)
            day = ((day + 0x10) & 0xf0);
         if (day > MonthDay[month])
            day = 0x01;
         break;
      }
      case 3:                 //Week Value Data Increment
      {
         week++;
         if (week >= 0x07 )
            week = 0;
         break;
      }
      case 4:                 //Hour Value data Increment
      {
         hour++;
         if ((hour & 0x0f) == 0x0a)
            hour = ((hour + 0x10) & 0xf0);
         if (hour >= 0x24 )
            hour = 0x00;
         break;
      }
      case 5:                 //Minut Value Data Increment
      {
         min++;
         if ((min & 0x0f) == 0x0a)
            min = ((min + 0x10) & 0xf0);
         if (min >= 0x60 )
            min = 0x00;
         break;
      }
      case 6:                 //Second Value Data Increment
      {
         sec++;
         if ((sec & 0x0f) == 0x0a)
            sec = ((sec + 0x10) & 0xf0);
         if (sec >= 0x60 )
            sec = 0x00;
         break;
      }
      default : break;
   }
   rtc_date_set();            //Update Value Data Set
}

void Down_value(int Dnval)
{
   switch(Dnval)
   {
      case 0:                 //Year Valu Data Decrement
      {
         year--;
         if (year == 0xff)
         {
            year = 0x99;
            year_h2--;
         }
         if ((year & 0x0f) == 0x0f)
            year = ((year & 0xf0) | 0x09);
         break;
      }
      case 1:                 //Month Value Data Decriment
      {
         month--;
         if (month == 0x00 )
            month = 0x12;
         if ((month & 0x0f) == 0x0f)
            month = ((month & 0xf0) | 0x09);
         break;
      }
      case 2:                 //Day Value Data Decriment
      {
         day--;
         if (day == 0)
            day = MonthDay[month];
         if ((day & 0x0f) == 0x0f)
            day = ((day & 0xf0) | 0x09);
         break;
      }
      case 3:                 //Week Value Data Decriment
      {
         week--;
         if (week == 0xff )
            week = 0x06;
         break;
      }
      case 4:                 //Hour Value data Decriment
      {
         hour--;
         if (hour == 0xff)
            hour = 0x23;
         if ((hour & 0x0f) == 0x0f)
            hour = ((hour & 0xf0) | 0x09);
         break;
      }
      case 5:                 //Minut Value Data Decriment
      {
         min--;
         if (min == 0xff)
            min = 0x59;
         if ((min & 0x0f) == 0x0f)
            min = ((min & 0xf0) | 0x09);
         break;
      }
      case 6:                 //Second Value Data Up
      {
         sec--;
         if (sec == 0xff)
            sec = 0x59;
         if ((sec & 0x0f) == 0x0f)
            sec = ((sec & 0xf0) | 0x09);
         break;
      }
      default : break;
   }
   rtc_date_set();            //Update Value Data Set
}

void main()
{
   setup_adc_ports(NO_ANALOGS);
   setup_adc(ADC_OFF);
   setup_psp(PSP_DISABLED);
   // TIMER Initialize //
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);

   // I/O Port Initialize //

   // TODO: USER CODE!!
   year_h2 = 20;
   output_high(LED_Red);            // LED_Red 'OFF'

   // LCD Initialize //
   lcd_init();                      //LCD Initialize
   lcd_clear();                     //LCD ALL CLEAR

   ///// Opening Message Disp Routine /////
   printf(LCD_data, "Gucci Tech  V3.5");
   LCD_cmd(0xc0);                   // move to line 2
   printf(LCD_data, "Date & Timer    ");
   delay_ms(2000);                  // Opening Message Disp Time
   lcd_clear();                     // LCD ALL CLEAR & Cursor Home Position

   // RTC Vdd Voltage Down Check! //
   i2c_start();                     // START Condition
   i2c_write(RTC_WAddr);            // Wite Mode SlaveWriteAddress
   i2c_write(0x02);                 // Second Address
   i2c_start();                     // START Condition
   i2c_write(RTC_RAddr);            // Read Mode SlaveReadAddress
   sec = (i2c_read(1)) & 0x80;      // Second Value
   i2c_stop();                      // STOP Condition

   // Year & Date & Week & Time First Data Set //
   if ((sec & 0b10000000) == 0x80)  //VL-bit='1'?
   {
      year_h2 = 20;                 //First Data set
      year=0x08; month=0x86; week=0x02; day=0x17; hour=0x12; min=0x00; sec=0x00;
      rtc_control_init();           //RTC-8564NB Initialize
      rtc_date_set();               //First Value Data Set
      rtc_date_read();
   }
   else
   {
      void time_disp2();            // RTC-8564NB First Data & Time Display
   }

   // 'Temperature' & 'Humidity' First Data measurement & Display //
      sht_sampling();
      sht_disp();

   output_low(LED_Red);             // LED_Red 'ON'
   delay_ms(300);
   output_high(LED_Red);            // LED_Red 'OFF'

   while(True)
   {
      time_disp1();                 //Now Date & Time Display
      if ((century == 1) && (year == 0x01))
         year_h2++;

      // 'Temperature' & 'Humidity' First Data measurement & Display //
      if (one_secf == 1)
      {
         sht_sampling();
         sht_disp();
         one_secf = 0;
      }

      // Year & Date & Week & Time Data Edit //
      if (~input(Edit_SW) == 1)     //'EDIT' Switch 'ON' Check!?
      {
         delay_ms(5);               //Chataring Cnacel Delay Time
         if (~input(Edit_SW) == 1)  //'EDIT' Switch 'ON' Check!?
         {
            lcd_cmd(0x0F);          // display ON,Cursor ON,Blink ON
            lcd_cmd(0x83);          // LCD-RAM-Address='3'
            UpDn_sw_cnt = 0;        // Up-Down Switch Counter Buffer Clear
            RL_sw_cnt = 0;          // Right-Left Switch Counter Buffer Clear
            while(~input(Edit_SW) == 1){  // EDIT Switch 'OFF' Check!?
               delay_ms(5);
            }
            delay_ms(5);
            while(True)
            {
               if (~input(Right_SW) == 1)    // Right Switch 'ON' Check!?
               {
                  RL_sw_cnt++;
                  if (RL_sw_cnt > 6)
                     RL_sw_cnt = 0;
                  change_1 = 1;              // Data Chenge! Flag1 '1'
               }
               if (~input(Left_SW) == 1)     // Left Switch 'ON' Check!?
               {
                  if (RL_sw_cnt == 0)
                     RL_sw_cnt = 6;
                  else
                     RL_sw_cnt--;
                  change_1 = 1;              // Data Chenge! Flag1 '1'
               }
               if (~input(Up_SW) == 1)       // Up Switch 'ON' Check!?
               {
                  Up_value(RL_sw_cnt);
                  change_1 = 1;              // Data Change! Flag1 '1'
               }
               if (~input(Down_SW) == 1)     // Down Switch 'ON' Check!?
               {
                  Down_value(RL_sw_cnt);
                  change_1 = 1;              // Data Change! Flag1 '1'
               }
               if (~input(EDIT_SW) == 1)     // EDIT Switch 'ON' Check!?
                  break;

               While ((~input_a() & 0b00110111) != 0){ //ALL Switch 'OFF'!?
                  delay_ms(10);              //Chataring Cnacel Delay Time
               }
                  delay_ms(10);              //Chataring Cnacel Delay Time

               if (change_1 == 1)
               {
                  time_disp1();              //Now Date & Time Display!
                  lcd_cmd(CurPosi[RL_sw_cnt]);
                  change_1 = 0;
               }
               else
                  change_1 = 0;

            }
            lcd_cmd(0x00);                   //LCD 1-Line Head
            lcd_cmd(0x0C);                   //display ON,Cursor OFF,Blink OFF
            While ((~input_a() & 0b00110111) != 0){ //ALL Switch 'OFF'!?
               delay_ms(10);                 //Chataring Cnacel Delay Time
            }
               delay_ms(10);                 //Chataring Cnacel Delay Time
         }
      }
   }
}
