/* SHT1x Humidity & Temperature Sensor

   2009/08/16 SHT Library

   RA4   SHT_DAT ---> RE1
   RA5   SHT_SCK ---> RE0



//#use  fast_io(E)

*/

void SHT_Trans_Start();
void SHT_Trans(int data);
void SHT_data_recive();

static unsigned long rdata_x;
static unsigned int crcdata;

// Delay Time
#define DLTIMEu 1      // Wait Time '1us'

// CRC Data //
int const crc_table[256] = {
0, 49, 98, 83, 196, 245,166, 151,185, 136,219, 234,125, 76, 31, 46, 67, 114,33, 16,
135, 182,229, 212,250, 203,152, 169,62, 15, 92, 109,134, 183,228, 213,66, 115,32, 17,
63, 14, 93, 108,251, 202,153, 168,197, 244,167, 150,1, 48, 99, 82, 124, 77, 30, 47,
184, 137,218, 235,61, 12, 95, 110,249, 200,155, 170,132, 181,230, 215,64, 113,34, 19,
126, 79, 28, 45, 186, 139,216, 233,199, 246,165, 148,3, 50, 97, 80, 187, 138,217, 232,
127, 78, 29, 44, 2, 51, 96, 81, 198, 247,164, 149,248, 201,154, 171,60, 13, 94, 111,
65, 112,35, 18, 133, 180,231, 214,122, 75, 24, 41, 190, 143,220, 237,195, 242,161, 144,
7, 54, 101, 84, 57, 8, 91, 106,253, 204,159, 174,128, 177,226, 211,68, 117,38, 23,
252, 205,158, 175,56, 9, 90, 107,69, 116,39, 22, 129, 176,227, 210,191, 142,221, 236,
123, 74, 25, 40, 6, 55, 100, 85, 194, 243,160, 145,71, 118,37, 20, 131, 178,225, 208,
254, 207,156, 173,58, 11, 88, 105,4, 53, 102, 87, 192, 241,162, 147,189, 140,223, 238,
121, 72, 27, 42, 193, 240,163, 146,5, 52, 103, 86, 120, 73, 26, 43, 188, 141,222, 239,
130, 179,224, 209,70, 119,36, 21, 59, 10, 89, 104,255, 206,157, 172};

// SHT1x Control signal

// *******************************
// DATA Line = Low, SCK Line = Low
// *******************************
void DL_SL()
{
   output_e(0b00000000);   // DATA-'L' , SCK-'L'
   delay_us(DLTIMEu);      // Wait '1us'
}

// *******************************
// DATA Line = High, SCK Line = Low
// *******************************
void DH_SL()
{
   output_e(0b00000010);   // DATA-'H' , SCK-'L'
   delay_us(DLTIMEu);      // Wait '1us'
}

// *******************************
// DATA Line = Low, SCK Line = High
// *******************************
void DL_SH()
{
   output_e(0b00000001);   // DATA-'L' , SCK-'H'
   delay_us(DLTIMEu);      // Wait '1us'
}

// *******************************
// DATA Line = High, SCK Line = High
// *******************************
void DH_SH()
{
   output_e(0b00000011);   // DATA-'H' , SCK-'H'
   delay_us(DLTIMEu);      // Wiat '1us'
}

// *******************************
// "Transmission Start" sequence
// *******************************
void SHT_Trans_Start()
{
   // PORT-E initial value //
   output_e(0xFF);
   SET_TRIS_E( 0b00000000 );    // DATA(RE1) = 'output'
   DH_SL();             // DATA-'H' , SCK-'L'
   DH_SH();             // DATA-'H' , SCK-'H'
   DL_SH();             // DATA-'L' , SCK-'H'
   DL_SL();             // DATA-'L' , SCK-'L'
   DL_SH();             // DATA-'L' , SCK-'H'
   DH_SH();             // DATA-'H' , SCK-'H'
   DH_SL();             // DATA-'H' , SCK-'L'
   // Wait //
   delay_ms(10);
}

// *******************************
// SHT Command transmission
// *******************************
void SHT_Trans(int data)
{
   unsigned int i;

   // crc //
   crcdata = 0;
   crcdata ^= data;      // 'crcdata' XOR 'data'
   crcdata = crc_table[crcdata];

   // Command transmission //
   DH_SL();             // DATA-'H' , SCK-'L'
   for(i = 0 ; i < 8 ; i++)
   {
      if((data & 0b10000000) == 0) {
         DL_SL();       // DATA-'L' , SCK-'L'
         DL_SH();       // DATA-'L' , SCK-'H'
         DL_SL();       // DATA-'L' , SCK-'L'
      } else {
         DH_SL();       // DATA-'H' , SCK-'L'
         DH_SH();       // DATA-'H' , SCK-'H'
         DH_SL();       // DATA-'H' , SCK-'L'
      }
      data <<= 1;
   }

   // The ACK signal is received. //
   SET_TRIS_E( 0b00000010 );    // DATA(RE1) = 'input'
   DH_SL();             // DATA-'H' , SCK-'L'
   DH_SH();             // DATA-'H' , SCK-'H'

   if (input(SHT_DAT) == 1)
   {
      // LCD Initialize //
      lcd_clear();      // LCD ALL CLEAR & Cursor Home Position
      // Error Message //
      printf(LCD_data, "ERROR; SHT dose not     ");
      LCD_cmd(0xc0);    // move to line 2
      printf(LCD_data, "respond to the command. ");
/*
      // Error Message //
      printf(LCD_data, "ERROR;");
      LCD_cmd(0x80);    // LCD 1-Line Top
      printf(LCD_data, "SHT does not respond");
      LCD_cmd(0xc0);    // LCD 2-Line Top
      printf(LCD_data, " to the command.");
      LCD_cmd(0x94);    // LCD 3-Line Top

      restart_wdt();    // Watchdog Timer Reset
      sleep();          // SLEEP 'ON'
      reset_cpu();      // CPU 'RESET'
*/
   }
   DH_SL();             // DATA-'H' , SCK-'L'
}

// *******************************
// SHT data recived
// *******************************
void  SHT_data_recive()
{
   unsigned int   rdata1,rdata2;
   unsigned int   i;
   unsigned int   read_crc;

   // MSB 8bit of the temperature data is received. //
   rdata1 = 0;
   for(i = 0 ; i < 8 ; i++)
   {
      DH_SH();          // DATA-'H' , SCK-'H'
      rdata1 <<= 1;
      if(input(SHT_DAT) == 1) rdata1 ++;
      DH_SL();          // DATA-'H' , SCK-'L'
   }
   crcdata ^= rdata1;
   crcdata = crc_table[crcdata];

   // The ACK signal is transmitted. //
   SET_TRIS_E( 0b00000000 );    // DATA(RE1) = 'output'
   // ACK Puls //
   DL_SL();             // DATA-'L' , SCK-'L'
   DL_SH();             // DATA-'L' , SCK-'H'
   DL_SL();             // DATA-'L' , SCK-'L'

   SET_TRIS_E( 0b00000010 );    // DATA(RE1) = 'input'
   // LSB 8bit of the temperature data is received. //
   rdata2 = 0;
   for(i = 0 ; i < 8 ; i++)
   {
      DH_SH();         // DATA-'H' , SCK-'H'
      rdata2 <<= 1;
      if(input(SHT_DAT) == 1) rdata2 ++;
      DH_SL();         // DATA-'H' , SCK-'L'
   }
   crcdata ^= rdata2;
   crcdata = crc_table[crcdata];

   SET_TRIS_E( 0b00000000 );    // DATA(RE1) = 'output'
   // ACK Puls(CRC used) //
   DL_SL();             // DATA-'L' , SCK-'L'
   DL_SH();             // DATA-'L' , SCK-'H'
   DL_SL();             // DATA-'L' , SCK-'L'

   // The CRC signal is received. //
   SET_TRIS_E( 0b00000010 );    // DATA(RE1) = 'input'
   read_crc = 0;
   for(i = 0 ; i < 8 ; i ++)
   {
      DH_SH();          // DATA-'H' , SCK-'H'
      read_crc >>= 1;
      if(input(SHT_DAT) == 1) read_crc += 0b10000000;
      DH_SL();          // DATA-'H' , SCK-'L'
   }
   DH_SL();             // DATA-'H' , SCK-'L'

   // ACK Puls //
   DL_SL();             // DATA-'L' , SCK-'L'
   DH_SH();             // DATA-'H' , SCK-'H'
   DH_SL();             // DATA-'H' , SCK-'L'
   // CRC check //
   if (crcdata != read_crc)
   {
      // LCD Initialize //
      lcd_clear();      // LCD ALL CLEAR & Cursor Home Position
      // Error Message //
      printf(LCD_data, "ERROR; Calc CRC:%02X",crcdata);
      LCD_cmd(0xc0);    // LCD 2-Line Top
      printf(LCD_data, "Read CRC : %02X",read_crc);
 
/*
      // Error Message //
      printf(LCD_data, "ERROR;");
      LCD_cmd(0x80);      // LCD 1-Line Top
      printf(LCD_data, "CRC Error!");
      LCD_cmd(0xc0);      // LCD 2-Line Top
      printf(LCD_data, "Calc CRC : %02X",crcdata);
      LCD_cmd(0x94);      // LCD 3-Line Top
      printf(LCD_data, "Read CRC : %02X",read_crc);
      LCD_cmd(0xd4);      // LCD 4-Line Top

      restart_wdt();      // Watchdog Timer Reset
      sleep();         // SLEEP 'ON'
      reset_cpu();      // CPU 'Reset'
*/
   }
   // MSB and LSB of the temperature data are stored in 16bit variable. //
   rdata_x = (long)rdata1 * 256 + (long)rdata2;
   // Wait //
   delay_ms(50);
}
