/* LCD Library Customize (lcd_lib_G1.c)
   2009/07/17 By GucciTech / Chojiro Higashiguchi

   Liquid Crystal Dsiplay PWB24227-CEM

   LCD Disp Control Library
   The internal organs function is as follows. 
      lcd_init()    ----- LCD Initialize
      lcd_cmd(cmd)  ----- Command Out
      lcd_data(chr) ----- 1 Character Disp Out
      lcd_clear()   ----- All Clear

   LCD Interface Use PORTD
   Pin Layout
   RS    RD1
   R/W   GND
   E     RD0
   DB4   RD4
   DB5   RD5
   DB6   RD6
   DB7   RD7
*/


//////// Data Out Sub Function
void lcd_out(int code, int flag)
{
  output_D((code & 0xF0) | (input_D() & 0x0F));
  if (flag == 0)
    output_high(LCD_RS);   //For the indicative data
  else
    output_low(LCD_RS);    //For the command data
  delay_cycles(1);         //NOP
  output_high(LCD_E);      //strobe out
  delay_cycles(2);         //NOP
  output_low(LCD_E);       //reset strobe
}
//////// 1 Character Disp Function
void lcd_data(int asci)
{
   lcd_out(asci, 0);       //High rank 4-bit power output
   lcd_out(asci<<4, 0);    //Subordinate position 4-bit power output
   delay_us(40);           //40us Wait
}
/////// Command Out Function
void lcd_cmd(int cmd)
{
   lcd_out(cmd, 1);        //High rank 4-bit power output
   lcd_out(cmd<<4, 1);     //Subordinate position 4-bit power output
   delay_ms(2);            //2msec Wait
}
/////// All Clear Function
void lcd_clear()
{
   lcd_cmd(0x01);          //Initialize Command OUT
   delay_ms(15);           //15msec Wait
}
/////// Initialize Function
void lcd_init()
{
   set_tris_D(LCD_mode);   //Mode Set
   delay_ms(15);
   lcd_out(0x30, 1);       //8bit mode set
   delay_ms(5);
   lcd_out(0x30, 1);       //8bit mode set
   delay_us(100);
   lcd_out(0x30, 1);       //8bit mode set
   delay_us(100);
   lcd_out(0x20, 1);       //4bit mode set
   delay_us(100);
   LCD_cmd(0x28);          // DL=0 4bit Mode ,2Line,5*7dots,
   LCD_cmd(0x08);          // display off Disp=Cursor=Blink=0
   LCD_cmd(0x0C);          // display on Disp=ON,Cursor=Blink=OFF
   LCD_cmd(0x06);          // entry mode set I/D=1(Increment) S=0
   LCD_cmd(0x02);          // Cursor Home
}
